! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

module srctool_test



  !-----------------------------------------------------------------------------
  ! Author: T. Dwelly (dwelly@mpe.mpg.de)
  !
  ! First version: 2018-20-21
  !-----------------------------------------------------------------------------
  !
  !-----------------------------------------------------------------------------
  ! Summary:
  !
  ! Module used to manage srctool's test routines
  !
  !-----------------------------------------------------------------------------
  !

  use iso_fortran_env ! a more standard replacement for numkind
!  use param          ! standard eROSITA module handling parameter/logging/error interface
  use ero_general    ! standard eROSITA module handling flag definitions etc

#ifdef _OPENMP
  use omp_lib
#endif


  use srctool_generic           ! generic module with some universal constants
!  use srctool_generic_fitswrap  ! generic fits file handling routines
  use srctool_generic_regions   ! generic region handling routines
  use srctool_generic_avign     ! generic analytic vignetting file handling routines
!  use srctool_datatypes         ! structs and datatypes
!  use srctool_misc              !
!  use srctool_instpar           !
!  use srctool_gti               !
!  use srctool_ebounds           !
!  use srctool_spec              !
  use srctool_att               !
!  use srctool_param             !
!  use srctool_lc                !
!  use srctool_badpix            !
!  use srctool_pattcal           !
!  use srctool_psf               !
!  use srctool_rmf               !
!  use srctool_arf               !
!  use srctool_detmap            !
!  use srctool_deadcor           !
!  use srctool_cal               !

  implicit none     ! insist on explicit typing

  !some private constants
  character(len=*), private, parameter :: modulename='SRCTOOL_TEST'



  public  :: srctool_test_harness

contains

  !-------------------------------------------------------------------------------
  !-------------------------------------------------------------------------------
  !--------------------------- Main test routine ---------------------------------
  !-------------------------------------------------------------------------------
  !-------------------------------------------------------------------------------
  ! Launch tests of each module and of generic functionality
  subroutine srctool_test_harness(instlist, ierr )
    character(len=*), parameter :: subroutinename='srctool_test_harness'
    type(instlist_type), intent(inout) :: instlist
    integer(kind=int32), intent(out)   :: ierr


!    ! local variables
!    type(src_type) :: src
!    !  this will hold the output spectra (if required)
!    type(speclist_type)         :: speclist
!    !  this will hold the output arf files (if required)
!    type(arflist_type)          :: arflist
!    !  this will hold the output rmf files (if required)
!    type(rmflist_type)          :: rmflist
!    !  this will hold the output lcs (if required)
!    type(lclist_type)           :: lclist

    character(len=LSTR_MAX) :: str_buffer
    character(len=STR_MAX) :: str_buffer2
    integer(kind=int32) :: clock_count, clock_count_src, clock_rate ! clock_count0
!    integer(kind=int32) :: i,j
!    real(kind=real64) :: x,y,v,e,oa
    ierr = ERROR_LEVEL_OK
    ! init the clock
    call system_clock(clock_count_src, clock_rate)


    str_buffer2 = "  1bcd   87587  ghvbv 587587   565667            565e4556  "
    call wrslog(subroutinename, "pre-squash:  '" // trim(str_buffer2) // "'" )
    call squash_string(str_buffer2)
    call wrslog(subroutinename, "post-squash: '" // trim(str_buffer2) // "'" )

    !----------------------------------------------------------
    ! suppress warnings
    write(str_buffer, '(A)') trim(instlist%inst(1)%name)


    call srctool_regions_test_harness(ierr)

    call srctool_att_test_harness(ierr)

    call avign_test_harness(ierr)


    if (ierr > ERROR_LEVEL_OK) then
      call wrserr(subroutinename, ierr, &
        "Error when testing srctool")
    else
      call wrslog(subroutinename, &
        "All tests passed successfully!")
    endif
    !----------------------------------------------------------




    !----------------------------------------------------------
    ! report results
    !----------------------------------------------------------

    !----------------------------------------------------------
    ! report processing time
    call system_clock(clock_count)
    write (str_buffer, '(F12.5)') real((clock_count - clock_count_src), real32)/real(clock_rate, real32)
!    call wrschatty(subroutinename, &
!        PROCTIME_PREFIX // "------------------ Finished Test: total proc time: " &
!        // trim(str_buffer) // "s -----------------------")
    call wrschatty(subroutinename, &
        PROCTIME_PREFIX // " Total time for tests: " // trim(str_buffer) // "s" )

    !----------------------------------------------------------

  end subroutine srctool_test_harness

  !-------------------------------------------------------------------------------
  !-------------------------------------------------------------------------------
  !--------------------------- Main test routine ---------------------------------
  !-------------------------------------------------------------------------------
  !-------------------------------------------------------------------------------
end module srctool_test

